<?php

/**
 * @file
 * Contains \Drupal\image_base64_formatter\Plugin\Field\FieldFormatter\ImageBase64Formatter.
 */

namespace Drupal\image_base64_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Form\FormStateInterface;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;
use Drupal\Core\Render\Markup;

/**
 * Plugin implementation of the 'image_base64_formatter'.
 *
 * @FieldFormatter(
 *   id = "image_base64",
 *   label = @Translation("Image Base64"),
 *   field_types = {
 *     "image"
 *   }
 * )
 */
class ImageBase64Formatter extends ImageFormatter {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The image style entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $imageStyleStorage;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
        'image_style' => '',
        'image_link' => '',
        'image_display' => '',
      ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    $image_styles = image_style_options(FALSE);
    $description_link = Link::fromTextAndUrl(
      $this->t('Configure Image Styles'),
      Url::fromRoute('entity.image_style.collection')
    );
    $element['image_style'] = [
      '#title' => $this->t('Image style'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('image_style'),
      '#empty_option' => $this->t('None (original image)'),
      '#options' => $image_styles,
      '#description' => $description_link->toRenderable() + [
          '#access' => $this->currentUser->hasPermission('administer image styles'),
        ],
    ];

    $link_types = [
      'content' => $this->t('Content'),
      'file' => $this->t('File'),
    ];
    $element['image_link'] = [
      '#title' => $this->t('Link image to'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('image_link'),
      '#empty_option' => $this->t('Nothing'),
      '#options' => $link_types,
    ];

    $display_types = [
      'content' => $this->t('Image Source'),
      'file' => $this->t('CSS background Source'),
    ];
    $element['image_display'] = [
      '#title' => $this->t('Show as'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('image_display'),
      '#empty_option' => $this->t('Base64 String'),
      '#options' => $display_types,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('Render Image Base64');

    $image_styles = image_style_options(FALSE);
    unset($image_styles['']);

    $image_style_setting = $this->getSetting('image_style');
    if (isset($image_styles[$image_style_setting])) {
      $summary[] = $this->t('Image style: @style', ['@style' => $image_styles[$image_style_setting]]);
    }
    else {
      $summary[] = $this->t('Original image');
    }

    $link_types = array(
      'content' => $this->t('Linked to content'),
      'file' => $this->t('Linked to file'),
    );
    // Display this setting only if image is linked.
    $image_link_setting = $this->getSetting('image_link');
    if (isset($link_types[$image_link_setting])) {
      $summary[] = $link_types[$image_link_setting];
    }

    $display_types = [
      'content' => $this->t('Show as Image Source'),
      'file' => $this->t('Show as CSS background Source'),
    ];
    // Display this setting only if show_image_as is set up.
    $image_display_setting = $this->getSetting('image_display');
    if (isset($display_types[$image_display_setting])) {
      $summary[] = $display_types[$image_display_setting];
    }

    return $summary;
  }

  /**
  * {@inheritdoc}
  */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $images = $this->getEntitiesToView($items, $langcode);

    // Early opt-out if the field is empty.
    if (empty($images)) {
      return $elements;
    }

    $url = NULL;
    $image_link_setting = $this->getSetting('image_link');
    // Check if the formatter involves a link.
    if ($image_link_setting === 'content') {
      $entity = $items->getEntity();
      if (!$entity->isNew()) {
        $url = $entity->toUrl();
      }
    }
    elseif ($image_link_setting === 'file') {
      $link_file = TRUE;
    }

    /** @var \Drupal\image\ImageStyleInterface $image_style */
    $image_style_setting = $this->getSetting('image_style');
    $image_style = $this->imageStyleStorage->load($image_style_setting);
    $image_display = $this->getSetting('image_display');

    /** @var \Drupal\file\FileInterface[] $images */
    foreach ($images as $delta => $image) {
      $image_uri = $image->getFileUri();
      $image_type = $image->getMimeType();

      $absolute_path = $image_style ?
        $this->imageStyleStorage->load($image_style->getName())->buildUrl($image_uri) :
        Url::fromUri(file_create_url($image_uri))->getUri();

      // decode image in base64
      $image_file = file_get_contents($absolute_path);
      $base_64_image = base64_encode($image_file);
      $base_64_data = "data:$image_type;base64,$base_64_image";

      // The image.factory service will check if our image is valid.
      $image_media = \Drupal::service('image.factory')->get($absolute_path);
      if ($image_media->isValid()) {
        $width = $image_media->getWidth();
        $height = $image_media->getHeight();
      }
      else {
        $width = $height = NULL;
      }

      switch ($image_display) {
        case 'content':
          $markup = '<img src="' . $base_64_data . '" ';
          if (isset($width, $height)) {
            $markup .= 'width="' . $width . '" height="' . $height . '" ';
          }
          $markup .= '/>';
          $markup = Markup::create(render($markup));
          break;
        case 'file':
          $markup = "url('$base_64_data')";
          break;
        default:
          $markup = $base_64_data;
      }
      $elements[$delta] = ['#markup' => $markup];

      // Add cacheability metadata from the image and image style.
      $cacheability = CacheableMetadata::createFromObject($image);
      if ($image_style) {
        $cacheability->addCacheableDependency(CacheableMetadata::createFromObject($image_style));
      }
      $cacheability->applyTo($elements[$delta]);
    }

    return $elements;
  }

}
